// Direct granules extruder Project
// More on: www.HomoFaciens.de

$fn = 100;

M3_hole = 3.5;
M3_nut_e = 6.3;
M4_hole = 4.5;
M5_hole = 5.5;
M6_hole = 6.5;
M5_nut_e = 8.79 + 0.3;
M6_nut_e = 11.05 + 0.3;
M8_nut_e = 14.38+1.5;

bb8_do = 22 + 0.5;
bb8_h = 7;

bb10_do = 30 + 0.5;
bb10_h = 9;

motor_e = 42;
motor_l = 40;
motor_m = 31;
motor_shaft = 5.5;
motor_shaft_shift = 4.9;

motor_plate_x = 42;
motor_plate_y = 70;

//Ball bearing
bb_do = 22;
bb_di = 8;
bb_w = 7;


// Mount grid
mg_dis = 15;
mg_d = 5.2;

// hotend mount holes
hot_m_d = 3;
hot_m_dis = 28;

// Copper pipe dimensions
cp_d = 15;

// Heater cartridge dimensions
heater_d = 6;
heater_dis = 15;
heater_z_shift = -4;

// Temperature sensor dimensions
temperature_z_shift = 3;
temperature_x_shift = 8;
temperature_d = 2;  

// Cold end fan
fan_m_dis = 55;


cut = 0;
assembled = 1;

if (assembled == 1){
  difference(){
    union(){

      hopper();

      translate([7.5, -50, -4+34])rotate([90, 0, 0]){
        Zonestar_mount();
      }
      
      translate([0, 0, 0])rotate([0, 180, 0]){
        hopper_mount(0);
      }      

//      translate([0, 0, 80])rotate([0, 0, 0]){
//        motor_plate();
//      }
      
      translate([0, 0, 80])rotate([180, 0, 0]){
        gear_plate_top();
      }

      translate([0, 0, 57])rotate([180, 0, 0]){
        gear_plate_bottom();
      }

      translate([0, 0, 50])rotate([180, 0, 0]){
        screw_coupler();
      }
      
      translate([0, 0, 60]){
        main_gear();
        translate([-43, 0, -11])
        stepper_gear();
      }
      
    }
    if(cut == 1){
      translate([0,25,0]){
        cube(size=[100,50,200], center=true);
      }
    }
  }
}
else{
  difference(){
    union(){
//      Zonestar_mount();
//      screw_coupler(M8_nut_e);
//      hopper_mount(0); // Top 
//      hopper_mount(1);  // Bottom
//      hopper();
//      main_gear();
//      stepper_gear();
//      gear_plate_top();
//      gear_plate_bottom();
//      spacer();
      spacer(9,8); // Stepper motor
//      spacer(5, 12, 4); // Gear mount plates
    }
    if(cut == 1){
      translate([0,25,0]){
        cube(size=[100,50,200], center=true);
      }
    }
  }
}


module spacer(s_h = 13, s_do=15, s_di=M3_hole){
  
  difference(){
    translate([0, 0, s_h/2])
    cylinder(d=s_do, h=s_h, center=true);
    translate([0, 0, s_h/2])
    cylinder(d=s_di, h=s_h+0.1, center=true);
  }
}

module main_gear(){
  mirror([0, 0, 1])
  translate([0, 0, -1])
  gear(teeth=70,h=1, shaft=8.5, mod=3.1, s=0.98);
  translate([0, 0, 1])
  gear(teeth=70,h=4, shaft=8.5, mod=3.1, s=1.0);
}

module stepper_gear(){
  sg_h = 10;
  sg_d = 20;
  
  difference(){
    union(){
      translate([0, 0, sg_h/2])
      cylinder(d=sg_d, h=sg_h, center=true);
      translate([0, 0, sg_h])
      gear(teeth=14,h=7, shaft=5, mod=3.1, s=1.0);
    }
    translate([0, 0, sg_h/2])
    cylinder(d=5, h=sg_h+0.1, center=true);
    
    translate([0, 0, sg_h/2])
    rotate([90, 0, 0])
    cylinder(d=M3_hole, h=sg_d*1.1, center=true);
    
    translate([0, 5.5, 4])
    cube(size=[5.7, 3, sg_h], center=true);
    
  }
}




module gear_plate_bottom(){
  mp_l = motor_plate_x + 12;
  mp_w = motor_plate_y + 12;
  mp_wall = 2;
  mp_h = bb8_h+2;
  stepper_pos = -36;
//  translate([0, 0, 70])
  difference(){
    union(){
      hull(){
        roundedRect([mp_l, mp_w, mp_wall], 5);
        translate([0, 0, mp_h/2])
        cylinder(d=bb8_do+2*mp_wall, h=mp_h, center=true);
      }
      // Mount holes hopper
      for(i=[0:1:1])mirror([i,0,0])
      for(j=[0:1:1])mirror([0,j,0]){
        translate([motor_plate_x/2, motor_plate_y/2, 2.5])
        cylinder(d=10, h=5, center=true);
      }
    }
//    translate([0, 0, mp_wall])
//    roundedRect([motor_e+2, motor_e+2, mp_h], 5);
    // Mount holes hopper
    for(i=[0:1:1])mirror([i,0,0])
    for(j=[0:1:1])mirror([0,j,0]){
      translate([motor_plate_x/2, motor_plate_y/2, mp_h/2])
      cylinder(d=M3_hole, h=mp_h+0.1, center=true);
    }

    
    // Center hole
    translate([0, 0, mp_h/2])
    cylinder(d=bb8_do-5, h=mp_h+0.1, center=true);
    translate([0, 0, mp_h/2+mp_wall])
    cylinder(d=bb8_do, h=mp_h, center=true);
    
    
  }
  
}


module gear_plate_top(){
  mp_l = motor_plate_x + 12;
  mp_w = motor_plate_y + 12;
  mp_wall = 2;
  mp_h = bb8_h+2;
  stepper_pos = -43;
//  translate([0, 0, 70])
  difference(){
    union(){
      roundedRect([mp_l, mp_w, mp_wall], 5);
      translate([0, 0, mp_h/2])
      cylinder(d=bb8_do+2*mp_wall, h=mp_h, center=true);
      // Stepper plate
      translate([stepper_pos, 0, 0])
      roundedRect([motor_e+15, motor_e+6, mp_wall], 5);
      for(i=[0:1:1])mirror([0, i, 0])
      hull(){
        translate([-17, motor_e/2+2, 0.05])
        cube(size=[85, mp_wall, 0.1], center=true);
        translate([-17, motor_e/2+2, mp_h-0.05])
        cube(size=[60, mp_wall, 0.1], center=true);
      }
    }
//    translate([0, 0, mp_wall])
//    roundedRect([motor_e+2, motor_e+2, mp_h], 5);
    // Mount holes hopper
    for(i=[0:1:1])mirror([i,0,0])
    for(j=[0:1:1])mirror([0,j,0]){
      translate([motor_plate_x/2, motor_plate_y/2, mp_h/2])
      cylinder(d=M3_hole, h=mp_h+0.1, center=true);
    }

    
    // Center hole
    translate([0, 0, mp_h/2])
    cylinder(d=bb8_do-5, h=mp_h+0.1, center=true);
    translate([0, 0, mp_h/2+mp_wall])
    cylinder(d=bb8_do, h=mp_h, center=true);
    
    // Stepper mpunt holes
    translate([stepper_pos, 0, 0]){
      hull(){
        translate([5, 0, 0])
        cylinder(d=22.5, h=mp_h+0.1, center=true);        
        translate([-5, 0, 0])
        cylinder(d=22.5, h=mp_h+0.1, center=true);        
      }
      for(i=[0:1:1])mirror([i,0,0])
      for(j=[0:1:1])mirror([0,j,0]){
        translate([motor_m/2, motor_m/2, mp_h/2])
        hull(){
          translate([5, 0, 0])
          cylinder(d=M3_hole, h=mp_h+0.1, center=true);
          translate([-5, 0, 0])
          cylinder(d=M3_hole, h=mp_h+0.1, center=true);      
        }
      }
    }
    
  }
  
}

module hopper(){
  h_d1 = 15;
  h_d2 = 110;
  h_d3 = 55;
  h_shift = 15;
  h_h = 50;
  h_wall = 2;
  h_w = 55;
  
  difference(){
    
    union(){
      roundedRect([h_d3, h_d3+20, h_wall], 5);
      intersection(){
        hull(){
          translate([0, h_shift, h_h-0.05])
          cylinder(d=h_d2, h=0.1, center=true);
          translate([0, 0, 0.05])
          cylinder(d=h_d1, h=0.1, center=true);
        }
        cube(size=[h_w, motor_plate_y*2.1, h_h*2.1], center=true);
      }
      // Backmount
      translate([0, -motor_plate_y/2-2, h_h/2])
      rotate([90, 0, 0])
      roundedRect([motor_plate_x+50, h_h, h_wall], 5);
      
      // side walls
      for(i=[0:1:1])mirror([i,0,0])
      translate([motor_plate_x/2, 0, h_h/2])
      cube(size=[h_wall, motor_plate_y, h_h], center=true);
      // Mount holes motor
      for(i=[0:1:1])mirror([i,0,0])
      for(j=[0:1:1])mirror([0,j,0]){
        translate([motor_plate_x/2, motor_plate_y/2, h_h/2])
        cylinder(d=11, h=h_h, center=true);
      }

      // mount holes cold end fan
      difference(){
        for(i=[0:1:1])mirror([0,i,0])
        translate([0, fan_m_dis/2, 5])
        rotate([0, 90, 0])
        cylinder(d=10, h=h_d3, center=true);
        cube(size=[40, h_d3*1.5, 50], center=true);
      }
    }

      // Backmount
      for(i=[0:1:1])mirror([i,0,0])
      translate([2.5*mg_dis, -motor_plate_y/2-2, 35])
      rotate([90, 0, 0])
      cylinder(d=M5_hole, h=h_wall*2.1, center=true);

    // auger cut
    cylinder(d=12, h=h_h+0.1, center=true);
    
    // Central cut
    difference(){
      translate([0, 0, h_wall*2])
      intersection(){
        hull(){
          translate([0, h_shift+0.7, h_h-0.05])
          cylinder(d=h_d2, h=0.1, center=true);
          translate([0, 0.7, 0.05])
          cylinder(d=h_d1, h=0.1, center=true);
        }
        a = h_w - 2 * h_wall;
        b = motor_plate_y*2.1;
        cube(size=[a, b, h_h*2.1], center=true);
      }
      
      // Mount holes motor
      for(i=[0:1:1])mirror([i,0,0])
      for(j=[0:1:1])mirror([0,j,0]){
        translate([motor_plate_x/2, motor_plate_y/2, h_h/2])
        cylinder(d=11, h=h_h, center=true);
      }
    }
    // Mount holes motor
    for(i=[0:1:1])mirror([i,0,0])
    for(j=[0:1:1])mirror([0,j,0]){
      translate([motor_m/2, motor_m/2, h_wall/2])
      cylinder(d=M3_hole, h=h_wall+0.1, center=true);
    }

    // Mount holes motor plate
    for(i=[0:1:1])mirror([i,0,0])
    for(j=[0:1:1])mirror([0,j,0]){
      translate([motor_plate_x/2, motor_plate_y/2, h_h/2])
      cylinder(d=M3_hole, h=h_h+0.1, center=true);
    }

    // mount holes cold end fan
    for(i=[0:1:1])mirror([0,i,0])
    translate([0, fan_m_dis/2, 5])
    rotate([0, 90, 0]){
      cylinder(d=M3_hole, h=h_d3+0.1, center=true);
      rotate([0, 0, 30])
      cylinder(d=M3_nut_e, h=h_d3-4, center=true, $fn=6);
    }

  }
  
}


module hopper_mount(lid = 1){
  hm_l = 55;
  hm_w = 24;
  hm_h = 7.5;
  hm_wall_1 = 2;
  hm_wall_2 = 5;
  hm_d = 55;
  hm_h2 = hm_h+hm_wall_1;
 
 
  difference(){
    union(){
      roundedRect([hm_l, motor_plate_y+15, hm_h2],2);
//      // Fan mount walls
//      for(i=[0:1:1])mirror([i,0,0])
//      translate([-hm_l/2, 0, 20])
//      rotate([0, 90, 0])
//      roundedRect([30, motor_plate_y, hm_wall_1],5);
      // Halfes mount points
      for(j=[0:1:1])mirror([0,j,0])
      for(i=[0:1:1])mirror([i,0,0]){
        translate([hm_l/2+2, motor_plate_y/2+2, hm_h2/2]){
          cylinder(d=10, h=hm_h2, center=true);
        }
        translate([hm_l/2+2, motor_plate_y/2-15, hm_h2/2]){
          cylinder(d=10, h=hm_h2, center=true);
        }
      }

      
    }
    
    // Halfes mount holes
    for(j=[0:1:1])mirror([0,j,0])
    for(i=[0:1:1])mirror([i,0,0]){
      translate([13, motor_plate_y/2-7.5, hm_h2/2]){
        cylinder(d=M3_hole, h=hm_h2+0.1, center=true);
        if(lid == 0)
        translate([0, 0, -2])
        hull(){
          cylinder(d=M3_hole, h=hm_h2, center=true);
          cylinder(d=M3_nut_e, h=hm_h2-4, center=true, $fn=6);
        }
      }
      translate([hm_l/2+2, motor_plate_y/2+2, hm_h2/2]){
        cylinder(d=M3_hole, h=hm_h2+0.1, center=true);
        if(lid == 0)
        translate([0, 0, -2])
        hull(){
          cylinder(d=M3_hole, h=hm_h2, center=true);
          cylinder(d=M3_nut_e, h=hm_h2-4, center=true, $fn=6);
        }
      }
      translate([hm_l/2+2, motor_plate_y/2-15, hm_h2/2]){
        cylinder(d=M3_hole, h=hm_h2+0.1, center=true);
        if(lid == 0)
        translate([0, 0, -2])
        hull(){
          cylinder(d=M3_hole, h=hm_h2, center=true);
          cylinder(d=M3_nut_e, h=hm_h2-4, center=true, $fn=6);
        }
      }
    }
    
    // Copper pipe cut
    translate([0, 0, cp_d/2+hm_wall_1])
    rotate([0, 90, 0])
    hull(){
      cylinder(d=cp_d+0.5, h=hm_l+0.1, center=true);
      translate([-30, 0, 0])
      cylinder(d=cp_d+0.5, h=hm_l+0.1, center=true);
    }
    
    // Center hole to hopper
    scale([1, 1, 1])
    cylinder(d=11.5, h=hm_h+hm_wall_1+0.1, center=true);

    if(lid == 1)
    translate([0, 0, -0.1])
    roundedRect([hm_l*2.5, hm_w*4.5, 5], 2);

    // Mount holes printer
    for(i=[0:1:1])mirror([i, 0, 0])
    translate([mg_dis*1.5, 0, cp_d/2+hm_wall_1])
    rotate([90, 0, 0]){
      cylinder(d=mg_d, h=motor_plate_y+50, center=true);
      cube(size=[15+50*lid,20,cp_d+15], center=true);
    }
    
    
    // Mount holes hopper
    for(i=[0:1:1])mirror([i,0,0])
    for(j=[0:1:1])mirror([0,j,0]){
      translate([motor_plate_x/2, motor_plate_y/2, hm_h2/2]){
        cylinder(d=M3_hole, h=hm_h2+0.1, center=true);
        translate([0, 0, 2])
        cylinder(d=M3_nut_e, h=hm_h2, center=true, $fn=6);
      }
    }

  }
}


module screw_coupler(nut=M8_nut_e, nut_h=6.5, insert=0){
  sc_d = 24;
  sc_shaft_h = 20;
  sc_wall = 6;
  sc_h = sc_shaft_h + sc_wall + nut_h;
  sc_shaft_d = 8.0;
  sc_shaft_shift = motor_shaft_shift;
  
  lid_wall = 4;
  lid_h = nut_h + sc_wall + lid_wall + 4;
  lid_do = sc_d + 2 * lid_wall + 1;
  lid_di = sc_d + 1;
  
  
  // Lid
  translate([30, 0, 0])
  difference(){
    translate([0, 0, lid_h/2]){
      cylinder(d=lid_do, h=lid_h, center=true);
    }

    translate([0, 0, lid_wall + lid_h/2]){
      cylinder(d=lid_di, h=lid_h, center=true);
    }
    cylinder(d=8, h=lid_h+0.1, center=true);

    // lid screw cut
    translate([0, 0, lid_wall+nut_h+sc_wall/2])
    rotate([90, 0, 0]){
      cylinder(d=M3_hole, h=lid_do+0.1, center=true);
      translate([0, 0, lid_do/2])
      cylinder(d=M3_nut_e, h=5, center=true, $fn=6);
      translate([0, 0, (3-lid_do)/2])
      cylinder(d2=M3_hole, d1=6, h=3, center=true);
    }
    
  }

  // Coupler
  difference(){
    translate([0, 0, sc_h/2]){
      cylinder(d=sc_d, h=sc_h, center=true);
    }
    
    //Center nut hole
    translate([0, 0, sc_h-nut_h/2+0.1]){
      cylinder(d=nut, h=nut_h, center=true, $fn=6);
    }

    // Shaft cut
    translate([0, 0, sc_shaft_h/2-0.1]){
      cylinder(d=sc_shaft_d, h=sc_shaft_h, center=true);
    }
    
    // lid screw cut
    translate([0, 0, sc_shaft_h+sc_wall/2])
    rotate([90, 0, 0])
    cylinder(d=M3_hole, h=sc_d+0.1, center=true);
    
    
    // Fixing nut cut
    translate([0, 0, sc_shaft_h/2])
    rotate([90, 0, 0])
    cylinder(d=M3_hole, h=sc_d+0.1, center=true);

    translate([0, -7.5, 0])
    cube(size=[5.7, 3, sc_shaft_h+8], center=true);
    
  }

}



module Zonestar_mount(){
  Zm_l = 80;
  Zm_w = 90;
  Zm_wall = 3;
    
  // Linear bearings
  lb_dis = 44;
  lb_dis_x = 45;
  lb_d = 15.5;
  lb_center = 15;
  
  difference(){
    union(){
      roundedRect([Zm_l, Zm_w, Zm_wall], 5);
      for(i=[0:1:1])mirror([0,i,0])
      translate([0, lb_dis / 2, 0]){
        hull(){
          translate([0, 0, lb_center])
          rotate([0, 90, 0])
          cylinder(d=lb_d+2*Zm_wall, h=Zm_l-10, center=true);
          translate([0, 0, 1])
          cube(size=[Zm_l-10, lb_d+2*Zm_wall, 0.1], center=true);
        }
      }
    }

    // Mount grid
    for(j=[0:1:1])mirror([0,j,0]){
      for(i=[0:1:5])
      translate([(i-3)*mg_dis, 5, Zm_wall/2]){
        cylinder(d=mg_d, h=Zm_wall*1.1, center=true);
      }
      for(i=[0:1:5])
      translate([(i-3)*mg_dis, 38, Zm_wall/2]){
        cylinder(d=mg_d, h=Zm_wall*1.1, center=true);
      }
    }

    // Linear bearings cut
    for(i=[0:1:1])mirror([0,i,0])
    translate([0, lb_dis / 2, 0]){
      translate([0, 0, lb_center])
      rotate([0, 90, 0])
      cylinder(d=lb_d, h=Zm_l, center=true);
    }
    
    // Screws linear bearings
    for(j=[0:1:1])mirror([j,0,0])
    for(i=[0:1:1])mirror([0,i,0])
    translate([0, lb_dis / 2, 0]){
      translate([lb_dis_x/2, 0, lb_center-5]){
        cylinder(d=M3_hole, h=21.5, center=true);
        cylinder(d=M3_nut_e, h=12, center=true, $fn=6);
      }
    }
  }
   
  
}


module gear(teeth=10,h=5, shaft=M3_hole, mod=3.5, s=1.0){
  alpha = 180/teeth;
  r=(mod/2.0)/tan(alpha);
  
  x = cos(alpha) * r;
  t_r = mod/sqrt(3);
  t_h = mod*sqrt(3)/2.0;
  
  difference(){
    linear_extrude(height = h, scale=s){
      union(){
        for(i=[0:1:teeth-1]){
          rotate([0, 0, i*360/teeth]){  
            translate([x+(t_h-t_r),0,0]){
              circle(r=t_r, $fn=3);
            }
          }
        }
        rotate([0, 0, alpha])
        circle(r=r, $fn=teeth);
      }
    }
    translate([0, 0, h/2]){
      cylinder(d=shaft, h=h*1.1, center=true);
    }
    if(shaft==-1){
      translate([0, 0, h/2]){
        difference(){
          cylinder(d=motor_shaft, h=h*1.1, center=true);
          translate([0, motor_shaft_shift, 0]){
            cube(size=[motor_shaft, motor_shaft, 16], center=true);
          }
        }
      }
    }
  }
  
}


module roundedRect(size, radius, edges=[1, 1, 1, 1], vscale=[1,1], shift=0){
  x = size[0];
  y = size[1];
  z = size[2];

  translate([0, -shift*2, 0]){
    linear_extrude(height=z, scale=[vscale[0], vscale[1]]){
      translate([0, shift*2, 0]){
        hull(){
            // place 4 circles in the corners, with the given radius
            if(edges[0]){
              translate([(-x/2)+(radius), (-y/2)+(radius), 0])
              circle(r=radius);
            }
            else{
              translate([(-x/2)+(radius/2), (-y/2)+(radius/2), 0])
              square(size=[radius, radius], center=true);
            }
            if(edges[1]){
              translate([(x/2)-(radius), (-y/2)+(radius), 0])
              circle(r=radius);
            }
            else{
              translate([(x/2)-(radius/2), (-y/2)+(radius/2), 0])
              square(size=[radius, radius], center=true);
            }

            if(edges[2]){
              translate([(-x/2)+(radius), (y/2)-(radius), 0])
              circle(r=radius);
            }
            else{
              translate([(-x/2)+(radius/2), (y/2)-(radius/2), 0])
              square(size=[radius, radius], center=true);
            }

            if(edges[3]){
              translate([(x/2)-(radius), (y/2)-(radius), 0])
              circle(r=radius);
            }
            else{
              translate([(x/2)-(radius/2), (y/2)-(radius/2), 0])
              square(size=[radius, radius], center=true);
            }
          }
       }
    }
  }
}